/*---------------------------------------------------------------------------*
 |          Copyright (c) 2002-2010 PDFlib GmbH. All rights reserved.        |
 +---------------------------------------------------------------------------+
 |    This software may not be copied or distributed except as expressly     |
 |    authorized by PDFlib GmbH's general license agreement or a custom      |
 |    license agreement signed by PDFlib GmbH.                               |
 |    For more information about licensing please refer to www.pdflib.com.   |
 *---------------------------------------------------------------------------*/

/* $Id: plop.java,v 1.58.2.7 2017/02/22 13:07:26 rjs Exp $
 *
 * PLOP Java class
 */

package com.pdflib;

/** PDFlib PLOP (PDF Linearization, Optimization, Protection)
    Note that this is only a syntax summary. For complete information
    please refer to the PLOP API reference manual which is available
    in the PDF file PLOP-manual.pdf in the PLOP distribution.
    @author Rainer Schaaf
    @version 5.2
*/

public final class plop implements IpCOS {

    // The initialization code for loading the PLOP shared library.
    // The library name will be transformed into something platform-
    // specific by the VM, e.g. libplop.so or plop.dll.

    static {
	try {
	    System.loadLibrary("plop");
	} catch (UnsatisfiedLinkError e) {
	    System.err.println(
	"Cannot load the PLOP shared library/DLL for Java.\n" +
	"Make sure to properly install the native PLOP library.\n\n" +
	"For your information, the current value of java.library.path is:\n" +
	 System.getProperty("java.library.path") + "\n");

	    throw e;
	}
    }

    // ------------------------------------------------------------------------
    // public functions
    /** Add recipient for encryption with certificate security.
        @return  -1 on error, and 1 otherwise.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final int add_recipient(String optlist)
    throws PLOPException
    {
	return PLOP_add_recipient(plop, optlist);
    }

    /** Close the input and output documents.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final void close_document(int doc, String optlist)
    throws PLOPException
    {
	PLOP_close_document(plop, doc, optlist);
    }

    /** Convert an arbitrary encoded string to a Unicode string in several formats.
        @return  The converted Unicode string.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final String convert_to_unicode(String inputformat, byte[] inputstring, String optlist)
    throws PLOPException
    {
	return PLOP_convert_to_unicode(plop, inputformat, inputstring, optlist);
    }

    /** Create a PDF output document (which may be protected) in memory or on disk file.
        @return  -1 on error, and a document handle otherwise.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final int create_document(String filename, String optlist)
    throws PLOPException
    {
	return PLOP_create_document(plop, filename, optlist);
    }

    /** Create a named virtual read-only file from data provided in memory.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final void create_pvf(String filename, byte[] data, String optlist)
    throws PLOPException
    {
	PLOP_create_pvf(plop, filename, data, optlist);
    }

    /** Delete a named virtual file and free its data structures (but not the contents).
        @return  -1 if the virtual file exists but is locked, and 1 otherwise.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final int delete_pvf(String filename)
    throws PLOPException
    {
	return PLOP_delete_pvf(plop, filename);
    }

    /** Get the name of the API function which threw the last exception or failed.
        @return  The name of a PLOP API function.
    */
    public final String get_apiname()
    {
	return PLOP_get_apiname(plop);
    }

    /** Fetch full or partial buffer contents of the output document from memory.
        @return  A buffer containing output data.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final byte[] get_buffer()
    throws PLOPException
    {
	return PLOP_get_buffer(plop);
    }

    /** Get the descriptive text of the last thrown exception, or the reason of a failed function call.
        @return  A string describing the error, or an empty string if the last API call didn't cause any error.
    */
    public final String get_errmsg()
    {
	return PLOP_get_errmsg(plop);
    }

    /** Get the number of the last thrown exception, or the reason of a failed function call.
        @return  The exception's error number.
    */
    public final int get_errnum()
    {
	return PLOP_get_errnum(plop);
    }

    /** Query properties of a virtual file or the PDFlib Virtual File system (PVF).
        @return  The value of some file parameter as requested by keyword.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final double info_pvf(String filename, String keyword)
    throws PLOPException
    {
	return PLOP_info_pvf(plop, filename, keyword);
    }

    /** Open a PDF document (which may be protected) for processing.
        @return  -1 on error, and a document handle otherwise.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final int open_document(String filename, String optlist)
    throws PLOPException
    {
	return PLOP_open_document(plop, filename, optlist);
    }

    /** Get the value of a pCOS path with type number or boolean.
        @return  The numerical value of the object identified by the pCOS path.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final double pcos_get_number(int doc, String path)
    throws PLOPException
    {
	return PLOP_pcos_get_number(plop, doc, path);
    }

    /** Get the value of a pCOS path with type name, number, string, or boolean.
        @return  The unencrypted data contained in the stream or string.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final byte[] pcos_get_stream(int doc, String optlist, String path)
    throws PLOPException
    {
	return PLOP_pcos_get_stream(plop, doc, optlist, path);
    }

    /** Get the value of a pCOS path with type name, string, or boolean.
        @return  A string with the value of the object identified by the pCOS path.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final String pcos_get_string(int doc, String path)
    throws PLOPException
    {
	return PLOP_pcos_get_string(plop, doc, path);
    }

    /** Prepare signature options.
        @return  -1 on error, and 1 otherwise.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final int prepare_signature(String optlist)
    throws PLOPException
    {
	return PLOP_prepare_signature(plop, optlist);
    }

    /** Set one or more global options for PLOP.
        @exception com.pdflib.PLOPException
	PLOP output cannot be finished after an exception.
    */
    public final void set_option(String optlist)
    throws PLOPException
    {
	PLOP_set_option(plop, optlist);
    }

    /** Create a new PLOP context.
	@exception com.pdflib.PLOPException May throw an exception in case
	of memory shortage.
     */
    public plop() throws PLOPException
    {
	plop = PLOP_new();
    }

    /** Delete a PLOP context and release all its internal resources.
	This should be called for cleanup when processing is done,
	and after a PLOPException occurred.
        This method may also be called by the finalizer, but it is safe to
	issue multiple calls.
     */
    public final void delete()
    {
	PLOP_delete(plop);
	plop = (long) 0;
    }

    // ------------------------------------------------------------------------
    // IpCOS-specific functions
    
    /** Open PLOP input document via the IpCOS interface.
        @exception java.lang.Exception
     */
    public int pcos_open_document(String filename, String optlist) throws Exception
    {
        return open_document(filename, optlist);
    }
    
    /** Close PLOP input document via the IpCOS interface.
        @exception java.lang.Exception
     */
    public void pcos_close_document(int doc, String optlist) throws Exception
    {
        close_document(doc, optlist);
    }
    
    // ------------------------------------------------------------------------
    // private functions

    private long plop;

    protected final void finalize()
    {
	PLOP_delete(plop);
	plop = (long) 0;
    }

    private final static native long PLOP_new() throws PLOPException;
    private final static native void PLOP_delete(long jplop);

    private final static native int PLOP_add_recipient(long jplop, String joptlist) throws PLOPException;
    private final static native void PLOP_close_document(long jplop, int jdoc, String joptlist) throws PLOPException;
    private final static native String PLOP_convert_to_unicode(long jplop, String jinputformat, byte[] jinputstring, String joptlist) throws PLOPException;
    private final static native int PLOP_create_document(long jplop, String jfilename, String joptlist) throws PLOPException;
    private final static native void PLOP_create_pvf(long jplop, String jfilename, byte[] jdata, String joptlist) throws PLOPException;
    private final static native int PLOP_delete_pvf(long jplop, String jfilename) throws PLOPException;
    private final static native String PLOP_get_apiname(long jplop);
    private final static native byte[] PLOP_get_buffer(long jplop) throws PLOPException;
    private final static native String PLOP_get_errmsg(long jplop);
    private final static native int PLOP_get_errnum(long jplop);
    private final static native double PLOP_info_pvf(long jplop, String jfilename, String jkeyword) throws PLOPException;
    private final static native int PLOP_open_document(long jplop, String jfilename, String joptlist) throws PLOPException;
    private final static native double PLOP_pcos_get_number(long jplop, int jdoc, String jpath) throws PLOPException;
    private final static native byte[] PLOP_pcos_get_stream(long jplop, int jdoc, String joptlist, String jpath) throws PLOPException;
    private final static native String PLOP_pcos_get_string(long jplop, int jdoc, String jpath) throws PLOPException;
    private final static native int PLOP_prepare_signature(long jplop, String joptlist) throws PLOPException;
    private final static native void PLOP_set_option(long jplop, String joptlist) throws PLOPException;
}
